@@ -158,6 +158,14 @@ my $RFC_WHITELIST =  Lintian::Data->new(
         return qr/$_[0]/xms;
     });
 
+# prebuilt-file or forbidden copyright
+my $BAD_LINK_COPYRIGHT =  Lintian::Data->new(
+    'cruft/bad-link-copyright',
+    qr/\s*\~\~\s*/,
+    sub {
+        return qr/$_[1]/xms;
+    });
+
 my $MISSING_DIR_SEARCH_PATH
   =  Lintian::Data->new('cruft/missing-dir-search-path');
 
@@ -198,8 +206,13 @@ sub _get_license_check_file {
 
             my @keywordlist = split(/\s*\&\&\s*/, $keywords);
             if(scalar(@keywordlist) < 1) {
-                fail $syntaxerror, 'No keywords line', $.;
+                fail $syntaxerror, 'No keywords on line', $.;
             }
+            my @sentencelist = split(/\s*\|\|\s*/, $sentence);
+            if(scalar(@sentencelist) < 1) {
+                fail $syntaxerror, 'No sentence on line', $.;
+            }
+
             if($regex eq '') {
                 $regex = '.*';
             }
@@ -208,7 +221,7 @@ sub _get_license_check_file {
             }
             my %ret = (
                 'keywords' =>  \@keywordlist,
-                'sentence' => $sentence,
+                'sentence' => \@sentencelist,
                 'regex' => qr/$regex/xsm,
                 'firstregex' => qr/$firstregex/xsm,
             );
@@ -524,6 +537,13 @@ sub check_debian_dir {
     return;
 }
 
+# testset exception
+sub istestset {
+    my ($dirname) = @_;
+    return $dirname
+      =~ m{ (:?\A|/) t (?: est (?: s (?: et)?+ )?+ )?+ (?: \Z | /)}xsm;
+}
+
 # Check each file in the source package for problems.  By the time we get to
 # this point, we've already checked the diff and warned about anything added
 # there, so we only warn about things that weren't in the diff here.
@@ -545,6 +565,7 @@ sub find_cruft {
         my $basename = $entry->basename;
         my $dirname = $entry->dirname;
         my $file_info;
+        my $istestsetdir = istestset($dirname);
 
         if ($entry->is_dir) {
 
@@ -562,11 +583,7 @@ sub find_cruft {
             # "source-contains-quilt-control-dir" tag.
             next if $basename eq '.pc';
 
-            # Ignore files in test suites.  They may be part of the test.
-            next
-              if $basename=~ m{ \A t (?: est (?: s (?: et)?+ )?+ )?+ \Z}xsm;
-
-            if (not $warned->{$name}) {
+            if (not $istestsetdir and not $warned->{$name}) {
                 for my $rule (@directory_checks) {
                     if ($basename =~ /$rule->[0]/) {
                         tag "${prefix}-$rule->[1]", $name;
@@ -581,6 +598,7 @@ sub find_cruft {
             next ENTRY;
         }
         if ($entry->is_symlink) {
+            next ENTRY if $istestsetdir;
 
             # An absolute link always escapes the root (of a source
             # package).  For relative links, it escapes the root if we
@@ -594,6 +612,8 @@ sub find_cruft {
 
         # we just need normal files for the rest
         next ENTRY unless $entry->is_file;
+        # avoid lintian testset
+        next ENTRY if $source_pkg eq 'lintian' && $istestsetdir;
 
         # check non free file
         my $md5sum = $info->md5sums->{$name};
@@ -612,21 +632,8 @@ sub find_cruft {
 
         $file_info = $entry->file_info;
 
-        # warn by file type
-        foreach my $tag_filetype ($WARN_FILE_TYPE->all) {
-            my $warn_data = $WARN_FILE_TYPE->value($tag_filetype);
-            my $regtype = $warn_data->{'regtype'};
-            if($file_info =~ m{$regtype}) {
-                my $regname = $warn_data->{'regname'};
-                if($name =~ m{$regname}) {
-                    tag $tag_filetype, $name;
-                    if($warn_data->{'checkmissing'}) {
-                        check_missing_source($entry,$info,$name, $basename,
-                            $dirname,$warn_data->{'transform'});
-                    }
-                }
-            }
-        }
+        # check full text problem
+        full_text_check($source_pkg, $entry, $info, $name, $basename,$dirname);
 
         # waf is not allowed
         if ($basename =~ /\bwaf$/) {
@@ -658,6 +665,24 @@ sub find_cruft {
 
         }
 
+        next ENTRY if $istestsetdir;
+
+        # warn by file type
+        foreach my $tag_filetype ($WARN_FILE_TYPE->all) {
+            my $warn_data = $WARN_FILE_TYPE->value($tag_filetype);
+            my $regtype = $warn_data->{'regtype'};
+            if($file_info =~ m{$regtype}) {
+                my $regname = $warn_data->{'regname'};
+                if($name =~ m{$regname}) {
+                    tag $tag_filetype, $name;
+                    if($warn_data->{'checkmissing'}) {
+                        check_missing_source($entry,$info,$name, $basename,
+                            $dirname,$warn_data->{'transform'});
+                    }
+                }
+            }
+        }
+
         # here we check old upstream specification
         # debian/upstream should be a directory
         if (   $name eq 'debian/upstream'
@@ -746,14 +771,16 @@ sub find_cruft {
             }
             close($fd);
         }
-        full_text_check($source_pkg, $entry, $info, $name, $basename,$dirname);
     }
     return;
 }
 
 # try to check if source is missing
 sub check_missing_source {
-    my ($file, $info, $name, $basename, $dirname,$replacementspairref) = @_;
+    my ($file, $info, $name, $basename, $dirname,$replacementspairref,
+        $extratext)
+      = @_;
+    $extratext //= '';
 
     # do not check missing source for non free
     if($info->is_non_free) {
@@ -819,7 +846,7 @@ sub check_missing_source {
             }
         }
     }
-    tag 'source-is-missing', $name;
+    tag 'source-is-missing', $name, $extratext;
     return;
 }
 
@@ -937,26 +964,44 @@ sub _search_in_block0 {
         # now search hidden minified
         _linelength_test($entry, $info, $name, $basename, $dirname,$block);
     }
+    # search link rel header
+    if(index($block,' rel="copyright" ') > -1) {
+        my $href = $block;
+        $href =~ m,<link \s+
+                  rel="copyright" \s+
+                  href="([^"]+)" \s*/? \s*>,xmsi;
+        if(defined($1)) {
+            my $copyrighttarget = $1;
+            foreach my $badcopyrighttag ($BAD_LINK_COPYRIGHT->all) {
+                my $regex =  $BAD_LINK_COPYRIGHT->value($badcopyrighttag);
+                if($copyrighttarget =~ m{$regex}) {
+                    tag $badcopyrighttag, $name;
+                    last;
+                }
+            }
+        }
+    }
     return;
 }
 
 # warn about prebuilt javascript and check missing source
 sub _warn_prebuilt_javascript{
     my ($entry, $info, $name, $basename, $dirname,$linelength,$cutoff) = @_;
-    tag 'source-contains-prebuilt-javascript-object',
-      $name, 'line length is', int($linelength),
-      "characters (>$cutoff)";
+    my $extratext
+      =  'line length is '.int($linelength)." characters (>$cutoff)";
+    tag 'source-contains-prebuilt-javascript-object',$name,$extratext;
     # Check for missing source.  It will check
     # for the source file in well known directories
     check_missing_source($entry,$info,$name,$basename,$dirname,
-        [['(?i)\.js$','.debug.js'],['(?i)\.js$','-debug.js'],['','']]);
+        [['(?i)\.js$','.debug.js'],['(?i)\.js$','-debug.js'],['','']],
+        $extratext);
     return;
 }
 
 # detect if max line of block is > cutoff
 # return false if file is minified
 sub _linelength_test_maxlength {
-    my ($entry, $info, $name, $basename, $dirname, $block, $cutoff) = @_;
+    my ($block, $cutoff) = @_;
     while($block =~ /([^\n]+)\n?/g){
         my $linelength = length($1);
         if($linelength > $cutoff) {
@@ -973,9 +1018,7 @@ sub _linelength_test {
     my $line;
     my $nextblock;
 
-    ($linelength)
-      = _linelength_test_maxlength($entry, $info, $name, $basename, $dirname,
-        $block,INSANE_LINE_LENGTH);
+    ($linelength)= _linelength_test_maxlength($block,INSANE_LINE_LENGTH);
     # first check if line >  INSANE_LINE_LENGTH that is likely minification
     # avoid problem by recursive regex with longline
     if($linelength) {
@@ -1011,8 +1054,7 @@ sub _linelength_test {
     while(length($nextblock)) {
         # check line above > SAFE_LINE_LENGTH
         ($linelength,$line,$nextblock)
-          = _linelength_test_maxlength($entry, $info, $name, $basename,
-            $dirname, $nextblock,SAFE_LINE_LENGTH);
+          = _linelength_test_maxlength($nextblock,SAFE_LINE_LENGTH);
         # no long line
         unless($linelength) {
             return;
@@ -1364,7 +1406,16 @@ sub _license_check {
         unless(defined($$cleanedblock)) {
             $$cleanedblock = _clean_block($block);
         }
-        unless(index($$cleanedblock,$licenseproblemdata->{'sentence'}) > -1){
+
+        my $foundsentence = 0;
+        my @sentencelist =  @{$licenseproblemdata->{'sentence'}};
+        foreach my $sentence (@sentencelist) {
+            if(index($$cleanedblock,$sentence) > -1){
+                $foundsentence = 1;
+            }
+        }
+
+        unless($foundsentence) {
             next LICENSE;
         }
         my $regex
