@@ -169,13 +169,13 @@ my %VCS_CANONIFY = (
             $_[1] = 'vcs-git-uses-invalid-user-uri';
         }
         $_[0] =~ s{\Qhttp://git.debian.org/\E}
-                  {http://anonscm.debian.org/git/};
+                  {https://anonscm.debian.org/git/};
         $_[0] =~ s{\Qhttp://anonscm.debian.org/git/git/\E}
-                  {http://anonscm.debian.org/git/};
+                  {https://anonscm.debian.org/git/};
         $_[0] =~ s{\Qgit://git.debian.org/\E}
-                  {git://anonscm.debian.org/};
+                  {https://anonscm.debian.org/git/};
         $_[0] =~ s{\Qgit://anonscm.debian.org/git/\E}
-                  {git://anonscm.debian.org/};
+                  {https://anonscm.debian.org/git/};
     },
     hg      => sub {
         $_[0] =~ s{\Qhttp://hg.debian.org/\E}
@@ -726,7 +726,7 @@ sub run {
                     && $known_obsolete_emacs{$alternatives[0]->[0]});
 
                 for my $part_d (@alternatives) {
-                    my ($d_pkg, $d_march, $d_version, $d_arch, undef, $rest,
+                    my ($d_pkg, undef, $d_version, undef, undef, $rest,
                         $part_d_orig)
                       = @$part_d;
 
@@ -973,11 +973,10 @@ sub run {
                         && &$is_dep_field($field));
 
                     for my $part_d (@alternatives) {
-                        my ($d_pkg, $d_march, $d_version, $d_arch, $d_restr,
+                        my ($d_pkg, undef, $d_version, $d_arch, $d_restr,
                             $rest,$part_d_orig)
                           = @$part_d;
 
-                        my $negated = 0;
                         for my $arch (@{$d_arch->[0]}) {
                             if ($arch eq 'all' || !is_arch_or_wildcard($arch)){
                                 tag 'invalid-arch-string-in-source-relation',
@@ -1292,6 +1291,9 @@ sub run {
                 if (any { $_ and /\s/} @parts) {
                     tag 'vcs-field-has-unexpected-spaces', "vcs-$vcs", $uri;
                 }
+                if ($parts[0] =~ m%^(?:git|http)://%) {
+                    tag 'vcs-field-uses-insecure-uri', "vcs-$vcs", $uri;
+                }
             }
             if ($VCS_CANONIFY{$vcs}) {
                 my $canonicalized = $parts[0];
