@@ -171,8 +171,8 @@ my @FILE_PACKAGE_MAPPING = (
     ),
 );
 
-sub _detect_embeded_libraries {
-    my ($fname, $file, $pkg,$info) = @_;
+sub _detect_embedded_libraries {
+    my ($fname, $file, $pkg) = @_;
 
     # detect only in regular file
     unless($file->is_regular_file) {
@@ -638,7 +638,7 @@ sub run {
 
                 }
             }
-            # ---------------- arch-indep pkconfig
+            # ---------------- arch-indep pkgconfig
             elsif ($file->is_regular_file
                 && $fname
                 =~ m,^usr/(?:lib(/[^/]+)?|share)/pkgconfig/[^/]+\.pc$,) {
@@ -827,6 +827,59 @@ sub run {
                     and $fname !~ m,^usr/bin/(?:X11|mh)/,) {
                     tag 'subdir-in-usr-bin', $file;
                 }
+                # check old style config script
+                elsif ( $file->is_regular_file
+                    and $fname =~ m,-config$,
+                    and $script{$file}) {
+                    # try to find some indication of
+                    # config file (read only one block)
+                    my $fd = $file->open(':raw');
+                    my $sfd = Lintian::SlidingWindow->new($fd);
+                    my $block = $sfd->readwindow();
+                    # some common stuff found in config file
+                    if (
+                        $block
+                        and (  index($block,'flag')>-1
+                            or index($block,'/include/') > -1
+                            or index($block,'pkg-config')  > -1)
+                      ) {
+                        # ok old config style script tag it
+                        tag 'old-style-config-script',$file;
+                        my $multiarch = $info->field('multi-arch','');
+                        # could be ok but only if multi-arch: no
+                        if($multiarch ne 'no' or $arch eq 'all') {
+                            # check multi-arch path
+                            foreach my $archs ($MULTIARCH_DIRS->all) {
+                                my $madir= $MULTIARCH_DIRS->value($archs);
+                                if ($block =~ m{\W\Q$madir\E(\W|$)}xms){
+                             # allow files to begin by triplet if it match arch
+                                    if($file->basename =~ m{^\Q$madir\E}xms) {
+                                        next;
+                                    }
+                                    if($arch eq 'all') {
+                                         #<<< No perltidy - tag name too long
+                                        tag
+                                          'old-style-config-script-multiarch-path-arch-all',
+                                          $file,
+                                          'full text contains architecture specific dir',
+                                          $madir;
+                                         #>>>
+                                    } else {
+                                        #<<< No perltidy - tag name too long
+                                        tag
+                                          'old-style-config-script-multiarch-path',
+                                          $file,
+                                          'full text contains architecture specific dir',
+                                          $madir;
+                                        #>>>
+                                    }
+                                    last;
+                                }
+                            }
+                        }
+                    }
+                    close($fd);
+                }
             }
             # ---------------- /usr subdirs
             elsif ($type ne 'udeb' and $fname =~ m,^usr/[^/]+/$,)
@@ -1353,7 +1406,7 @@ sub run {
             }
 
             # ---------------- embedded libraries
-            _detect_embeded_libraries($fname, $file, $pkg, $info);
+            _detect_embedded_libraries($fname, $file, $pkg);
 
             # ---------------- embedded Feedparser library
             if (    $fname =~ m,/feedparser\.py$,
@@ -1374,7 +1427,7 @@ sub run {
                     and $file->basename eq 'defaultpresets.xml') {
                     # false positive
                 } else {
-                    detect_privacy_breach($info,$file);
+                    detect_privacy_breach($file);
                 }
             }
             # ---------------- fonts
@@ -1495,12 +1548,12 @@ sub run {
                     close($fd);
                     if ($mtime != 0) {
                         if ($isma_same && $file !~ m/\Q$arch\E/o) {
-                            tag 'gzip-file-is-not-multi-arch-same-safe', $file;
+                            tag 'gzip-file-is-not-multi-arch-same-safe',$file;
                         } else {
                             # see https://bugs.debian.org/762105
-                            my $diff = $file->timestamp - $changelog_timestamp;
-                            if ($diff >= 0) {
-                                tag 'package-contains-timestamped-gzip', $file;
+                            my $diff= $file->timestamp - $changelog_timestamp;
+                            if ($diff > 0) {
+                                tag 'package-contains-timestamped-gzip',$file;
                             }
                         }
                     }
@@ -1668,9 +1721,9 @@ sub run {
             }
             if ($fname =~ m,^usr/share/locale/([^/]+)/$,) {
                 # Without encoding:
-                my ($lwccode) = split(/[.@]/, $1);
+                my ($lwccode) = split(m/[.@]/, $1);
                 # Without country code:
-                my ($lcode) = split(/_/, $lwccode);
+                my ($lcode) = split(m/_/, $lwccode);
 
                 # special exception:
                 if ($lwccode ne 'l10n') {
@@ -2043,7 +2096,7 @@ sub detect_generic_privacy_breach {
 }
 
 sub detect_privacy_breach {
-    my ($info, $file) = @_;
+    my ($file) = @_;
     my %privacybreachhash;
 
     # detect only in regular file
