@@ -21,6 +21,27 @@ package Lintian::Path;
 use strict;
 use warnings;
 use parent qw(Class::Accessor::Fast);
+
+use constant {
+    TYPE_FILE      => 0x00_01_00_00,
+    TYPE_HARDLINK  => 0x00_02_00_00,
+    TYPE_DIR       => 0x00_04_00_00,
+    TYPE_SYMLINK   => 0x00_08_00_00,
+    TYPE_BLOCK_DEV => 0x00_10_00_00,
+    TYPE_CHAR_DEV  => 0x00_20_00_00,
+    TYPE_PIPE      => 0x00_40_00_00,
+    TYPE_OTHER     => 0x00_80_00_00,
+    TYPE_MASK      => 0x00_ff_00_00,
+
+    UNSAFE_PATH    => 0x01_00_00_00,
+    FS_PATH_IS_OK  => 0x02_00_00_00,
+    OPEN_IS_OK     => 0x06_00_00_00, # Implies FS_PATH_IS_OK
+    ACCESS_INFO    => 0x07_00_00_00,
+    # 0o6777 == 0xdff, which covers set[ug]id + sticky bit.  Accordingly,
+    # 0xffff should be more than sufficient for the foreseeable future.
+    OPERM_MASK     => 0x00_00_ff_ff,
+};
+
 use overload (
     '""' => \&_as_string,
     'qr' => \&_as_regex_ref,
@@ -81,17 +102,9 @@ Argument is a hash containing the data read from the index file.
 sub new {
     my ($type, $data) = @_;
     my $self = $data;
-    my $ftype = $data->{'type'};
+    my $ftype = $data->{'_path_info'};
     bless($self, $type);
-    # Use $data->{type} directly here.  The constructor is called
-    # often enough for this to take up a (small) measure amount of
-    # runtime.
-    if ($ftype eq '-' or $ftype eq 'h') {
-        $self->{'_is_open_ok'} = 1;
-        $self->{'_valid_path'} = 1;
-    } elsif ($ftype eq 'd') {
-        $self->{'_is_open_ok'} = 0;
-        $self->{'_valid_path'} = 1;
+    if ($ftype & TYPE_DIR) {
         for my $child ($self->children) {
             $child->_set_parent_dir($self);
         }
@@ -113,6 +126,14 @@ NB: If only numerical owner information is available in the package,
 this may return a numerical owner (except uid 0 is always mapped to
 "root")
 
+=cut
+
+sub owner {
+    my ($self) = @_;
+    return 'root' if not exists($self->{'owner'});
+    return $self->{'owner'};
+}
+
 =item group
 
 Returns the group of the path entry as a username.
@@ -121,22 +142,46 @@ NB: If only numerical owner information is available in the package,
 this may return a numerical group (except gid 0 is always mapped to
 "root")
 
+=cut
+
+sub group {
+    my ($self) = @_;
+    return 'root' if not exists($self->{'group'});
+    return $self->{'group'};
+}
+
 =item uid
 
 Returns the uid of the owner of the path entry.
 
-NB: If the uid is not available, undef will be returned.
+NB: If the uid is not available, 0 will be returned.
 This usually happens if the numerical data is not collected (e.g. in
 source packages)
 
+=cut
+
+sub uid {
+    my ($self) = @_;
+    return 0 if not exists($self->{'uid'});
+    return $self->{'uid'};
+}
+
 =item gid
 
 Returns the gid of the owner of the path entry.
 
-NB: If the gid is not available, undef will be returned.
+NB: If the gid is not available, 0 will be returned.
 This usually happens if the numerical data is not collected (e.g. in
 source packages)
 
+=cut
+
+sub gid {
+    my ($self) = @_;
+    return 0 if not exists($self->{'gid'});
+    return $self->{'gid'};
+}
+
 =item link
 
 If this is a link (i.e. is_symlink or is_hardlink returns a truth
@@ -154,19 +199,26 @@ NB: Even for symlinks, a leading "./" will be stripped.
 
 Returns the size of the path in bytes.
 
-NB: This is only well defined for files.
+NB: Only regular files can have a non-zero file size.
+
+=cut
+
+sub size {
+    my ($self) = @_;
+    return 0 if not exists($self->{'size'});
+    return $self->{'size'};
+}
 
 =item date
 
 Return the modification date as YYYY-MM-DD.
 
-=item operm
+=cut
 
-Returns the file permissions of this object in octal (e.g. 0644).
-
-NB: This is only well defined for file entries that are subject to
-permissions (e.g. files).  Particularly, the value is not well defined
-for symlinks.
+sub date {
+    my ($self) = @_;
+    return (split(' ', $self->{'date_time'}, 2))[0];
+}
 
 =item parent_dir
 
@@ -183,6 +235,15 @@ File::Basename::dirname.  The dirname will end with a trailing slash
 
 NB: Returns the empty string for the "root" dir.
 
+=cut
+
+sub dirname {
+    my ($self) = @_;
+    my $dir = $self->parent_dir;
+    return $dir->name if $dir;
+    return q{};
+}
+
 =item basename
 
 Returns the "filename" part of the name, similar basename(1) or
@@ -192,6 +253,22 @@ for the "root" dir - see below).
 
 NB: Returns the empty string for the "root" dir.
 
+=cut
+
+sub basename {
+    my ($self) = @_;
+    my $name = $self->name;
+    my $slash;
+    return $name if $name eq q{}; # Root dir
+    if (substr($name, -1, 1) eq '/') {
+        $slash = rindex($name, '/', length($name) - 2);
+    } else {
+        $slash = rindex($name, '/');
+    }
+    return $name if $slash == -1; # E.g. Top level-dirs
+    return substr($name, $slash+1);
+}
+
 =item faux
 
 Returns a truth value if this entry absent in the package.  This can
@@ -199,11 +276,22 @@ happen if a package does not include all intermediate directories.
 
 =cut
 
-Lintian::Path->mk_ro_accessors(
-    qw(name owner group link type uid gid
-      size date time operm parent_dir dirname basename
-      faux
-      ));
+Lintian::Path->mk_ro_accessors(qw(name link parent_dir faux));
+
+=item operm
+
+Returns the file permissions of this object in octal (e.g. 0644).
+
+NB: This is only well defined for file entries that are subject to
+permissions (e.g. files).  Particularly, the value is not well defined
+for symlinks.
+
+=cut
+
+sub operm {
+    my ($self) = @_;
+    return $self->{'_path_info'} & OPERM_MASK;
+}
 
 =item children
 
@@ -228,9 +316,7 @@ seconds since the start of Unix epoch in UTC.
 
 sub timestamp {
     my ($self) = @_;
-    my $date = $self->{'date'};
-    my $time = $self->{'time'};
-    return str2time("$date $time", 'GMT');
+    return str2time($self->{'date_time'}, 'GMT');
 }
 
 =item child(BASENAME)
@@ -302,11 +388,14 @@ symlinks, even if the symlink points to a file.
 
 =cut
 
-sub is_symlink { return $_[0]->type eq 'l'; }
-sub is_hardlink { return $_[0]->type eq 'h'; }
-sub is_dir { return $_[0]->type eq 'd'; }
-sub is_file { return $_[0]->type eq '-' || $_[0]->type eq 'h'; }
-sub is_regular_file  { return $_[0]->type eq '-'; }
+sub is_symlink { return $_[0]->{'_path_info'} & TYPE_SYMLINK ? 1 : 0; }
+sub is_hardlink { return $_[0]->{'_path_info'} & TYPE_HARDLINK ? 1 : 0; }
+sub is_dir { return $_[0]->{'_path_info'} & TYPE_DIR ? 1 : 0; }
+
+sub is_file {
+    return $_[0]->{'_path_info'} & (TYPE_FILE | TYPE_HARDLINK) ? 1 : 0;
+}
+sub is_regular_file { return $_[0]->{'_path_info'} & TYPE_FILE ? 1 : 0; }
 
 =item link_normalized
 
@@ -359,7 +448,7 @@ at least one bit denoting executability set (bitmask 0111).
 
 sub _any_bit_in_operm {
     my ($self, $bitmask) = @_;
-    return ($self->operm & $bitmask) ? 1 : 0;
+    return ($self->{'_path_info'} & $bitmask) ? 1 : 0;
 }
 
 sub is_readable   { return $_[0]->_any_bit_in_operm(0444); }
@@ -378,10 +467,10 @@ files.
 sub file_info {
     my ($self) = @_;
     my $file_info;
-    if (my $file_info = $self->{'_file_info'}) {
+    if ($file_info = $self->{'_file_info'}) {
         return $file_info;
     }
-    $file_info = $self->{'_fs_info'}->_file_info($self);
+    $file_info = $self->_fs_info->_file_info($self);
     $self->{'_file_info'} = $file_info;
     return $file_info;
 }
@@ -419,7 +508,7 @@ defined entry, for which L</is_dir> returns a truth value.
 sub fs_path {
     my ($self) = @_;
     my $path = $self->_collect_path();
-    $self->_check_access($path);
+    $self->_check_access();
     return $path if $self->resolve_path->is_dir;
     $self->_check_open($path);
     return $path;
@@ -436,7 +525,9 @@ Returns a truth value if the path may be opened.
 
 sub is_open_ok {
     my ($self) = @_;
-    return $self->{'_is_open_ok'} if exists($self->{'_is_open_ok'});
+    my $path_info = $self->{'_path_info'};
+    return 1 if ($path_info & OPEN_IS_OK) == OPEN_IS_OK;
+    return 0 if $path_info & ACCESS_INFO;
     eval {
         my $path = $self->_collect_path();
         $self->_check_open($path);
@@ -447,40 +538,47 @@ sub is_open_ok {
 
 sub _collect_path {
     my ($self) = @_;
-    return $self->{'_fs_info'}->_underlying_fs_path($self);
+    return $self->_fs_info->_underlying_fs_path($self);
+}
+
+sub _fs_info {
+    my ($self) = @_;
+    # Technically, this will look up the parent dir even if $self is a dir
+    # - though calling is_dir first is probably more expensive than just
+    #   blindly calling parent_dir
+    my $dir = $self->parent_dir // $self;
+    return $dir->{'_fs_info'};
 }
 
 sub _check_access {
-    my ($self, $path) = @_;
-    my $safe = 1;
-    if (exists($self->{'_valid_path'})) {
-        $safe = $self->{'_valid_path'};
-    } else {
-        my $resolvable = $self->resolve_path;
-        $safe = 0 if not $resolvable;
-    }
-    if (not $safe) {
-        $self->{'_valid_path'} = $self->{'_is_open_ok'} = 0;
+    my ($self) = @_;
+    my $path_info = $self->{'_path_info'};
+    return 1 if ($path_info & FS_PATH_IS_OK) == FS_PATH_IS_OK;
+    return 0 if $path_info & ACCESS_INFO;
+    my $resolvable = $self->resolve_path;
+    if (not $resolvable) {
+        $self->{'_path_info'} |= UNSAFE_PATH;
         # NB: We are deliberately vague here to avoid suggesting
         # whether $path exists.  In some cases (e.g. lintian.d.o)
         # the output is readily available to wider public.
         confess('Attempt to access through broken or unsafe symlink:'. ' '
               . $self->name);
     }
-    $self->{'_valid_path'} = 1;
+    $self->{'_path_info'} |= FS_PATH_IS_OK;
     return 1;
 }
 
 sub _check_open {
     my ($self, $path) = @_;
-    $self->_check_access($path);
+    $self->_check_access();
     # Symlinks can point to a "non-file" object inside the
     # package root
     if ($self->is_file or ($self->is_symlink and -f $path)) {
-        $self->{'_is_open_ok'} = 1;
+        $self->{'_path_info'} |= OPEN_IS_OK;
         return 1;
     }
-    $self->{'_is_open_ok'} = 0;
+    # Leave "_path_access" here as _check_access marks it either as
+    # "UNSAFE_PATH" or "FS_PATH_IS_OK"
     confess("Attempt to open non-file (e.g. dir or pipe): $self");
 }
 
@@ -623,7 +721,7 @@ sub resolve_path {
     my ($self, $path_str) = @_;
     my $current = $self;
     my (@queue, %traversed_links, $had_trailing_slash);
-    my $fs_info = $self->{'_fs_info'};
+    my $fs_info = $self->_fs_info;
 
     if (defined($path_str) and ref($path_str) ne q{}) {
         croak('resolve_path only accepts string arguments');
@@ -697,7 +795,7 @@ sub resolve_path {
     return $current;
 }
 
-### OVERLOADED OVERATORS ###
+### OVERLOADED OPERATORS ###
 
 # overload apparently does not like the mk_ro_accessor, so use a level
 # of indirection
